@@ -18,7 +18,7 @@
 #' @note Function will scrap two CRAN URLS. Works only with CRAN packages.
 #' Please as a courtesy to the R CRAN, don't overload their servers by constantly using this function.
 #' The base part of URL in the result is `https://cran.r-project.org/src/contrib/`.
-#' Results are cached for 1 hour with `memoise` package.
+#' Results are cached for 30 minutes with `memoise` package.
 #' @export
 #' @examples
 #' \dontrun{
@@ -32,7 +32,6 @@ pac_timemachine <- function(pac,
                             from = NULL,
                             to = NULL,
                             version = NULL) {
-  stopifnot(pac_isin(pac, "https://cran.rstudio.com/"))
   stopifnot(is.null(version) || (length(version) == 1 && is.character(version)))
   stopifnot(xor(
     !is.null(at) && inherits(at, "Date") && is.null(version),
@@ -40,6 +39,8 @@ pac_timemachine <- function(pac,
   ) ||
     all(c(is.null(at), is.null(from), is.null(to), is.null(version))) || (!is.null(version) && length(version) == 1 && is.character(version)))
 
+  if (isFALSE(pac_isin(pac, "https://cran.rstudio.com/"))) return(NA)
+
   result <- pac_archived(pac)
   cran_page <- pac_cran_recent(pac)
 
@@ -102,7 +103,7 @@ pac_cran_recent_raw <- function(pac) {
   }
 }
 
-pac_cran_recent <- memoise::memoise(pac_cran_recent_raw, cache = cachem::cache_mem(max_age = 60 * 60))
+pac_cran_recent <- memoise::memoise(pac_cran_recent_raw, cache = cachem::cache_mem(max_age = 30 * 60))
 
 pac_archived_raw <- function(pac) {
   base_archive <- sprintf("/src/contrib/Archive/%s/", pac)
@@ -141,4 +142,4 @@ pac_archived_raw <- function(pac) {
   result
 }
 
-pac_archived <- memoise::memoise(pac_archived_raw, cache = cachem::cache_mem(max_age = 60 * 60))
+pac_archived <- memoise::memoise(pac_archived_raw, cache = cachem::cache_mem(max_age = 30 * 60))
