@@ -1,4 +1,3 @@
-
 replaceNA <- function(vec, with) {
   vec[is.na(vec)] <- with
   vec
@@ -68,8 +67,19 @@ compareVersionsMin <- function(vec, na.rm = TRUE) {
   )
 }
 
-dir_size <- function(path, recursive = TRUE) {
+#' Size of the package
+#' @description size of package.
+#' @param path path to the directory. Default: `"."`
+#' @param recursive logical if to assess the dependencies recursively. Default: TRUE
+#' @return numeric size in bytes, to get MB ten divide by `10**6`.
+#' @export
+#' @examples
+#' \dontrun{
+#' cat(pacs::dir_size(system.file(package = "stats")) / 10**6, "MB")
+#' }
+dir_size <- function(path = ".", recursive = TRUE) {
   stopifnot(is.character(path))
+  stopifnot(is.logical(recursive))
   files <- list.files(path, full.names = T, recursive = recursive)
   vect_size <- sapply(files, function(x) file.size(x))
   size_files <- sum(vect_size)
@@ -107,7 +117,7 @@ pacs_base <- function(startup = FALSE) {
 }
 
 pacs_base_all_raw <- function() {
-  rownames(installed_packages(priority = "base"))
+  rownames(installed_packages(lib.loc = NULL, priority = "base"))
 }
 
 pacs_base_all <- memoise::memoise(pacs_base_all_raw)
@@ -146,7 +156,7 @@ installed_descriptions <- function(lib.loc, fields, deps = NULL) {
   res_agg[!is.na(res_agg$Package), ]
 }
 
-installed_agg_fun_raw <- function(lib.loc = NULL, fields) {
+installed_agg_fun_raw <- function(lib.loc, fields) {
   installed_df <- as.data.frame(installed_packages(lib.loc = lib.loc))
   installed_agg <- stats::aggregate(
     installed_df[, c("Version", fields), drop = FALSE],
@@ -156,22 +166,22 @@ installed_agg_fun_raw <- function(lib.loc = NULL, fields) {
   installed_agg
 }
 
-installed_agg_fun <- memoise::memoise(installed_agg_fun_raw, cache = cachem::cache_mem(max_age = 60 * 60))
+installed_agg_fun <- memoise::memoise(installed_agg_fun_raw, cache = cachem::cache_mem(max_age = 30 * 60))
 
 #' List Available Packages at CRAN-like Repositories
 #' @description available_packages returns a matrix of details corresponding to packages currently available at one or more repositories. The current list of packages is downloaded over the internet (or copied from a local mirror).
 #' @param repos character vector, the base URL(s) of the repositories to use. Default `pacs::biocran_repos()`
-available_packages <- function(repos = biocran_repos()) {
+available_packages <- function(repos) {
   tryCatch(available_packages_raw(repos = repos), error = function(e) NA)
 }
 
-available_packages_raw <- memoise::memoise(utils::available.packages, cache = cachem::cache_mem(max_age = 60 * 60))
+available_packages_raw <- memoise::memoise(utils::available.packages, cache = cachem::cache_mem(max_age = 30 * 60))
 
-installed_packages <- function(lib.loc = NULL, priority = NULL) {
+installed_packages <- function(lib.loc, priority = NULL) {
   installed_packages_raw(lib.loc = lib.loc, priority = priority)
 }
 
-installed_packages_raw <- memoise::memoise(utils::installed.packages, cache = cachem::cache_mem(max_age = 60 * 60))
+installed_packages_raw <- memoise::memoise(utils::installed.packages, cache = cachem::cache_mem(max_age = 30 * 60))
 
 extract_deps <- function(x) {
   splited <- stri_split_fixed(x, ",")
@@ -229,7 +239,7 @@ available_descriptions <- function(repos, fields, deps = NULL) {
   res_agg
 }
 
-available_agg_fun_raw <- function(repos = "https://cran.rstudio.com/", fields) {
+available_agg_fun_raw <- function(repos, fields) {
   available_df <- as.data.frame(available_packages(repos = repos))
   available_agg <- stats::aggregate(
     available_df[, c("Version", fields), drop = FALSE],
@@ -239,4 +249,20 @@ available_agg_fun_raw <- function(repos = "https://cran.rstudio.com/", fields) {
   available_agg
 }
 
-available_agg_fun <- memoise::memoise(available_agg_fun_raw, cache = cachem::cache_mem(max_age = 60 * 60))
+available_agg_fun <- memoise::memoise(available_agg_fun_raw, cache = cachem::cache_mem(max_age = 30 * 60))
+
+expand_dependency <- function(x) {
+  if (length(x) == 1) {
+    stopifnot(all(x %in% c("strong", "all", "most")))
+    switch(
+      x,
+      strong = c("Depends", "Imports", "LinkingTo"),
+      most = c("Depends", "Imports", "LinkingTo", "Suggests"),
+      all = c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances"),
+      c("Depends", "Imports", "LinkingTo")
+    )
+  } else {
+    stopifnot(all(x %in% c("Depends", "Imports", "Suggests", "LinkingTo", "Enhances")))
+    x
+  }
+}
